package internal

import (
	"bytes"
	"context"
	"encoding/json"
	"fmt"
	"gitea.cdlsxd.cn/sdk/plugin/proto"
	"io/ioutil"
	"net/http"
	"strings"
)

// 插件通信信息,若不对应则会报错panic
const (
	Tag         = "zltx_card_v1"
	Version     = 1
	CookieKey   = "zltx_card_v1"
	CookieValue = "zltx_card_v1"
)

type ZLTXCardV1Service struct{}

func (p *ZLTXCardV1Service) Order(ctx context.Context, request *proto.OrderRequest) (*proto.OrderResponse, error) {
	c, err := transConfig(request.Config)
	if err != nil {
		return nil, err
	}

	dctW, err := c.dctW()
	if err != nil {
		return nil, err
	}

	resp, err := dctW.Order(ctx, c.orderReq(request))
	if err != nil {
		return nil, err
	}
	if !resp.GetCode().IsSuccess() {
		return nil, fmt.Errorf(resp.Message)
	}

	return orderResp(request, resp), nil
}

func (p *ZLTXCardV1Service) Query(ctx context.Context, request *proto.QueryRequest) (*proto.QueryResponse, error) {
	c, err := transConfig(request.Config)
	if err != nil {
		return nil, err
	}

	dctW, err := c.dctW()
	if err != nil {
		return nil, err
	}

	resp, cardCode, err := dctW.Query(ctx, c.queryReq(request))
	if err != nil {
		return nil, err
	}
	if !resp.GetCode().IsSuccess() {
		return nil, fmt.Errorf(resp.Message)
	}

	return queryResp(request, resp, cardCode)
}

func (p *ZLTXCardV1Service) Notify(ctx context.Context, request *proto.NotifyRequest) (*proto.NotifyResponse, error) {
	c, err := transConfig(request.Config)
	if err != nil {
		return nil, err
	}

	httpHeaders := make(http.Header)
	if err = json.Unmarshal(request.Headers, &httpHeaders); err != nil {
		return nil, fmt.Errorf("headers Unmarshal err [%v]", err)
	}
	newHeaders := make(http.Header)
	for key, values := range httpHeaders {
		newKey := strings.Title(strings.ToLower(key))
		newHeaders[newKey] = values
	}

	req := &http.Request{
		Header: newHeaders,
		Body:   ioutil.NopCloser(bytes.NewBuffer(request.Body)),
	}

	dctW, err := c.dctW()
	if err != nil {
		return nil, err
	}

	resp, cardCode, err := dctW.Notify(ctx, req)
	if err != nil {
		return nil, err
	}

	return notifyResp(resp, cardCode)
}
